/*
 * Copyright (c) 2019, NVIDIA CORPORATION.  All rights reserved.
 *
 * NVIDIA CORPORATION and its licensors retain all intellectual property
 * and proprietary rights in and to this software, related documentation
 * and any modifications thereto.  Any use, reproduction, disclosure or
 * distribution of this software and related documentation without an express
 * license agreement from NVIDIA CORPORATION is strictly prohibited.
 */
 
#pragma once

#include <vector>
#include <stdint.h>

#include <nvrhi/nvrhi.h>

#include <imgui/imgui.h>

struct ImGui_NVRHI
{
    std::vector<uint8_t> vertexShaderBinary;
    std::vector<uint8_t> pixelShaderBinary;

    nvrhi::DeviceHandle renderer = nullptr;
    nvrhi::CommandListHandle m_commandList;

    nvrhi::ShaderHandle vertexShader;
    nvrhi::ShaderHandle pixelShader;
    nvrhi::InputLayoutHandle shaderAttribLayout;

    // the constant buffer data depends only on the display size
    // cache the display size we updated the buffer with to avoid touching it every frame
    ImVec2 lastDisplaySize = ImVec2(-1.f, -1.f);
    nvrhi::BufferHandle constantBuffer;

    nvrhi::TextureHandle fontTexture;
    nvrhi::SamplerHandle fontSampler;

    nvrhi::BufferHandle vertexBuffer;
    nvrhi::BufferHandle indexBuffer;

    nvrhi::BindingLayoutHandle bindingLayout;
    nvrhi::GraphicsPipelineDesc basePSODesc;

    nvrhi::unordered_map<nvrhi::ITexture*, nvrhi::FramebufferHandle> framebuffers;

    nvrhi::GraphicsPipelineHandle pso;
    nvrhi::unordered_map<nvrhi::ITexture*, nvrhi::BindingSetHandle> bindingsCache;

    std::vector<ImDrawVert> vtxBuffer;
    std::vector<ImDrawIdx> idxBuffer;

    bool init(nvrhi::DeviceHandle renderer);
    void setFontScaleFactor(float scale);
    bool beginFrame(float elapsedTimeSeconds);
    bool render(nvrhi::ITexture* renderTarget);
    void backbufferResizing();

private:
    bool reallocateBuffer(nvrhi::BufferHandle& buffer, size_t requiredSize, size_t reallocateSize, bool isIndexBuffer);

    bool compileShaders();
    bool createFontTexture(nvrhi::ICommandList* commandList);

    nvrhi::IFramebuffer* getFramebuffer(nvrhi::ITexture* renderTarget);
    nvrhi::IGraphicsPipeline* getPSO(nvrhi::IFramebuffer* fb);
    nvrhi::IBindingSet* getBindingSet(nvrhi::ITexture* texture);
    bool updateGeometry(nvrhi::ICommandList* commandList);
    void updateConstantBuffer(nvrhi::ICommandList* commandList);
};
